package com.muzhi.tianhe.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.malk.server.aliwork.YDConf;
import com.malk.server.aliwork.YDParam;
import com.malk.server.aliwork.YDSearch;
import com.malk.server.common.McR;
import com.malk.service.aliwork.YDClient;
import com.malk.utils.UtilEnv;
import com.malk.utils.UtilMap;
import com.muzhi.tianhe.service.ThTbService;
import com.muzhi.tianhe.util.PublicUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.lang.reflect.Array;
import java.util.*;
import java.util.stream.Collectors;

@Slf4j
@Service
public class ThTbServiceImpl implements ThTbService {

    @Autowired
    private TbApiService tbApiService;
    @Autowired
    private YDClient ydClient;

    private static String PROJECT_TAG_ID = "65b77b8f7bbdd92f48e6920d"; // 战略部分组

    @Override
    public void task(String taskId,String type,JSONObject data) {
        JSONObject task=tbApiService.getTasksInfo(taskId).getJSONObject(0);
        String projectId=task.getString("projectId");
        // 先判断项目分组
        JSONArray array=tbApiService.getProjectTag(projectId);
        if(array==null||array.size()<1){
            log.info("不属于战略部分组的项目，不处理");
            return;
        }
        if(!Arrays.asList("65b77b8f7bbdd92f48e6920d","664d5b10e6451d2386a49b8b","664d5b15a1a96d9b5815d6fd").contains(array.getString(0))){
            log.info("不属于战略部分组的项目，不处理");
            return;
        }
//        String tfsId=task.getString("tfsId");// 状态ID
//        JSONObject tfs=tbApiService.getProjectStateId(projectId,"","",tfsId).getJSONObject(0);
        // 任务类型判断
        String sfcId=task.getString("sfcId");
        List<JSONObject> list=tbApiService.getTasksType(projectId).toJavaList(JSONObject.class);
        Optional optional = list.stream().filter(item -> sfcId.equals(item.get("id"))).findAny();
        JSONObject taskType=((JSONObject) optional.get());
        String name=taskType.getString("name");
        String tfid=taskType.getString("taskflowId");
        if(type.equals("0")){
            if(name.contains("变更")){
                log.info("变更流程");
                taskChange(task,taskId);
            }else if(name.equals("年度指标")){
                log.info("年度指标");
                updateTaskState(task,tfid);
            }else{
                log.info("任务 {} 任务类型：{} 非期望类型任务 不处理",task.getString("content"),name);
            }
        }else if(type.equals("1")){
            // 任务自定义字段变更
            if(name.contains("级任务")){
                taskCustChange(task,data);
            }
        }else if(type.equals("2")){
            // 任务新增
            if(name.contains("变更")){
                log.info("新增变更流程任务");
                creatChangeTask(task,name);
            }
        }
    }

    @Override
    public void taskLink(String taskId, String linkedId) {
        JSONObject task=tbApiService.getTasksInfo(taskId).getJSONObject(0);
        String projectId=task.getString("projectId");
        // 先判断项目分组
        JSONArray array=tbApiService.getProjectTag(projectId);
        if(array==null||array.size()<1){
            log.info("不属于战略部分组的项目，不处理");
            return;
        }
        if(!Arrays.asList("65b77b8f7bbdd92f48e6920d","664d5b10e6451d2386a49b8b","664d5b15a1a96d9b5815d6fd").contains(array.getString(0))){
            log.info("不属于战略部分组的项目，不处理");
            return;
        }
    }

    public void updateTaskState(JSONObject task,String tfid){
        String projectId=task.getString("projectId");// 项目ID
        // 上级任务信息
        String parentTaskId=task.getString("parentTaskId");// 上级任务ID
        JSONObject parentTask=tbApiService.getTasksInfo(parentTaskId).getJSONObject(0);
        JSONObject parentTaskTfs=tbApiService.getProjectStateId(projectId,"","",parentTask.getString("tfsId")).getJSONObject(0);
        String parentTaskTfsName=parentTaskTfs.getString("name");
        String newTfsName;
        // 判断状态
        JSONObject taskTfs=tbApiService.getProjectStateId(projectId,"",tfid,task.getString("tfsId")).getJSONObject(0);
        String taskTfsName=taskTfs.getString("name");
        if(taskTfsName.equals("年度目标未达成")){
            newTfsName="年度目标未达成";
        }else if(taskTfsName.equals("年度目标达成")){
            JSONObject notTfs=tbApiService.getProjectStateId(projectId,"年度目标未达成",tfid,"").getJSONObject(0);
            String notId=notTfs.getString("id");
            String notZbID;
            JSONObject zbtf=tbApiService.getProjectTfid(projectId,"年度指标工作流").getJSONObject(0);
            if(tfid.equals(zbtf.getString("id"))){
                notZbID=notId;
            }else{
                JSONObject redZbTfs=tbApiService.getProjectStateId(projectId,"年度目标未达成",zbtf.getString("id"),"").getJSONObject(0);
                notZbID=redZbTfs.getString("id");
            }
            JSONArray array=tbApiService.getTasksByParent(parentTaskId);
            newTfsName="年度目标达成";
            int notNum=0;
            for (int i = 0; i < array.size(); i++) {
                JSONObject taskT=array.getJSONObject(i);
                if(notId.equals(taskT.getString("tfsId"))||notZbID.equals(taskT.getString("tfsId"))){
                    notNum++;
                }
            }
            if(notNum>=1){
                newTfsName="年度目标未达成";
            }
        }else{
            // 红绿灯
            // 循环获取其他同级任务状态，判断是否有更高级别的任务状态
            // 同级红灯状态ID
            taskStatusChange(task);// 同步到宜搭
            JSONObject redTfs=tbApiService.getProjectStateId(projectId,"红灯",tfid,"").getJSONObject(0);
            String redId=redTfs.getString("id");
            // 同级黄状态ID
            JSONObject yellowTfs=tbApiService.getProjectStateId(projectId,"黄灯",tfid,"").getJSONObject(0);
            String yellowId=yellowTfs.getString("id");
            // 指标任务状态
            String redZbId,yellowZbId;

            JSONObject zbtf=tbApiService.getProjectTfid(projectId,"年度指标工作流").getJSONObject(0);
            if(tfid.equals(zbtf.getString("id"))){
                redZbId=redId;
                yellowZbId=yellowId;
            }else{
                JSONObject redZbTfs=tbApiService.getProjectStateId(projectId,"红灯",zbtf.getString("id"),"").getJSONObject(0);
                redZbId=redZbTfs.getString("id");
                // 同级黄状态ID
                JSONObject yellowZbTfs=tbApiService.getProjectStateId(projectId,"黄灯",zbtf.getString("id"),"").getJSONObject(0);
                yellowZbId=yellowZbTfs.getString("id");
            }
            JSONArray array=tbApiService.getTasksByParent(parentTaskId);
            newTfsName="绿灯";
            int redNum=0,yellowNum=0;
            for (int i = 0; i < array.size(); i++) {
                JSONObject taskT=array.getJSONObject(i);
                if(redId.equals(taskT.getString("tfsId"))||redZbId.equals(taskT.getString("tfsId"))){
                    redNum++;
                }else if(yellowId.equals(taskT.getString("tfsId"))||yellowZbId.equals(taskT.getString("tfsId"))){
                    yellowNum++;
                }
            }
            if(yellowNum>=1){
                newTfsName="黄灯";
            }
            if(redNum>=1){
                newTfsName="红灯";
            }
        }

        if(newTfsName.equals(parentTaskTfsName)){
            return;
        }
        // 任务类型判断
        String parentSfcId=parentTask.getString("sfcId");
        List<JSONObject> list=tbApiService.getTasksType(projectId).toJavaList(JSONObject.class);
        Optional optional = list.stream().filter(item -> parentSfcId.equals(item.get("id"))).findAny();
        JSONObject parentTaskType=((JSONObject) optional.get());
        String parentTfid=parentTaskType.getString("taskflowId");

        update(projectId,parentTaskId,newTfsName,parentTfid);
        if(!PublicUtil.isNull(parentTask.getString("parentTaskId"))){
            updateTaskState(parentTask,parentTfid);
        }
    }

    public void taskChange(JSONObject task,String taskId){
        String projectId=task.getString("projectId");// 项目ID
        // 查询关联任务
        JSONArray linkArray=tbApiService.getTaskLinks(taskId);
        if(linkArray==null||linkArray.size()<1){
            log.info("任务{}没有关联任务",taskId);
            return;
        }
        // 查询项目自定义字段
        Map<String,String> projectCusts=new HashMap<>();
        List<JSONObject> list=tbApiService.getProjectCustomfield(projectId).toJavaList(JSONObject.class);
        list.forEach(item -> projectCusts.put(item.getString("name"),item.getString("id")));
        // 查询任务中的自定义字段值
        Map<String,JSONArray> taskCustValues=new HashMap<>();
        List<JSONObject> custs=task.getJSONArray("customfields").toJavaList(JSONObject.class);
        custs.forEach(item -> taskCustValues.put(item.getString("cfId"),item.getJSONArray("value")));

        String value=taskCustValues.get(projectCusts.get("变更后内容")).getJSONObject(0).getString("title");
        JSONObject idObj=taskCustValues.get(projectCusts.get("内容变更/取消")).getJSONObject(0);
        String linkedId=linkArray.getJSONObject(0).getString("linkedId");
        log.info("任务[{}]字段:{}[{}]数据修改为：{}",linkedId,idObj.getString("title"),idObj.getString("id"),value);

        JSONObject linkeTask=tbApiService.getTasksInfo(linkedId).getJSONObject(0);
        String linkSfcId=linkeTask.getString("sfcId");
        JSONObject linkTaskType=tbApiService.getTasksType(projectId,linkSfcId).getJSONObject(0);
        String linkTaskTypeName=linkTaskType.getString("name");

        String qzId;
        if("年度指标".equals(linkTaskTypeName)){
            switch (idObj.getString("id")){
                case "66581ea7d74759b91fab2806":
                    // 年度指标
                    tbApiService.updateTaskContent(linkedId,value);
                    return;
                case "665820165bf840a9d00f0a36":
                    // 指标权重
                    qzId=projectCusts.get("权重");
                    break;
                case "66581ea7d74759b91fab2809":
                    // Q1目标
                    qzId=projectCusts.get("Q1目标");
                    break;
                case "66581ea7d74759b91fab280a":
                    // Q2目标
                    qzId=projectCusts.get("Q2目标");
                    break;
                case "66581ea7d74759b91fab280b":
                    // Q3目标
                    qzId=projectCusts.get("Q3目标");
                    break;
                case "66581ea7d74759b91fab280c":
                    // Q4目标
                    qzId=projectCusts.get("Q4目标");
                    break;
                case "66581ebba1a96d9b58350495":
                    // 年度指标验收依据
                    qzId=projectCusts.get("验收依据");
                    break;
                case "66582147e30d72623e4108d3":
                    log.info("任务取消");
                    taskCancel(projectId,linkTaskType,linkedId);
                    return;
                default:
                    log.info("非期望字段，暂不处理");
                    return;
            }
        }else if("三级任务,二级任务,一级任务".contains(linkTaskTypeName)){
            switch (idObj.getString("id")){
                case "66602856c563f93072f7bc2d":
                    // 任务目标
                    tbApiService.updateTaskContent(linkedId,value);
                    return;
                case "664e9d5ae6451d2386a6d1bd":
                    // 任务权重
                    if("三级任务".equals(linkTaskTypeName)){
                        qzId=projectCusts.get("权重");
                        tbApiService.updateTaskCustomfield(linkedId,qzId,value);
                    }else{
                        log.info("非三级任务，不修改权重");
                        return;
                    }
                    break;
                case "664e9d5ae6451d2386a6d1be":
                    // 责任人
                    qzId=projectCusts.get("责任人");
                    JSONArray array=tbApiService.userSearch(value);
                    if(array==null||array.size()<1){
                        log.error("未查到该人员 查询条件：{},返回结果：{}",value,array);
                        return;
                    }
                    value=array.getJSONObject(0).getString("userId");
                    break;
                case "665820165bf840a9d00f0a38":
                    // 联席责任人
                    qzId=projectCusts.get("联席责任人");
                    JSONArray array2=tbApiService.userSearch(value);
                    if(array2==null||array2.size()<1){
                        log.error("未查到该人员 查询条件：{},返回结果：{}",value,array2);
                        return;
                    }
                    value=array2.getJSONObject(0).getString("userId");
                    break;
                case "66582147e30d72623e4108d3":
                    log.info("任务取消");
                    taskCancel(projectId,linkTaskType,linkedId);
                    return;
                default:
                    log.info("非期望字段，暂不处理");
                    return;
            }
        }else{
            log.info("关联任务的任务类型[{}]非期望值",linkTaskTypeName);
            return;
        }
        tbApiService.updateTaskCustomfield(linkedId,qzId,value);
    }


    public void taskCancel(String projectId,JSONObject taskType,String taskId){
        // 查询关联任务
        String tfid=taskType.getString("taskflowId");
        update(projectId,taskId,"取消",tfid);
        JSONArray sonTasks=tbApiService.getTasksByParent(taskId);
        if(sonTasks!=null&&sonTasks.size()>0){
            for (int i = 0; i < sonTasks.size(); i++) {
                String sonTaskId=sonTasks.getJSONObject(i).getString("id");
                JSONObject sonTaskType=tbApiService.getTasksType(projectId,sonTaskId).getJSONObject(0);
                taskCancel(projectId,sonTaskType,sonTaskId);
            }
        }

    }

    public void taskAdd(JSONObject task,String taskId,JSONObject newTaskType){
        String projectId=task.getString("projectId");
        // 查询项目自定义字段
        List<JSONObject> projectCoutList=tbApiService.getProjectCustomfield(projectId).toJavaList(JSONObject.class);
        Map<String,String> custIdMap=new HashMap();
        projectCoutList.forEach(item -> custIdMap.put(item.getString("name"),item.getString("id")));
        List<JSONObject> taskCoutlist=task.getJSONArray("customfields").toJavaList(JSONObject.class);
        Map<String,JSONArray> taskCustValueMap=new HashMap();
        taskCoutlist.forEach(item -> taskCustValueMap.put(item.getString("cfId"),item.getJSONArray("value")));

        JSONObject addTask=new JSONObject();
        addTask.put("projectId",projectId);// 必填
        addTask.put("content",taskCustValueMap.get(custIdMap.get("新任务目标")).getJSONObject(0).getString("title"));// 标题必填
        addTask.put("scenariofieldconfigId",newTaskType.getString("id"));// 任务类型id 业务必填
        if(!newTaskType.getString("name").contains("一级")){
            // 查询关联任务 用于确认父级任务
            JSONArray linkArray=tbApiService.getTaskLinks(taskId);
            if(linkArray==null||linkArray.size()<1){
                log.info("任务{}没有关联任务",taskId);
                return;
            }
            String linkedId=linkArray.getJSONObject(0).getString("linkedId");
            addTask.put("parentTaskId",linkedId);// 父任务ID
        }
        JSONArray array=new JSONArray();
        array.add(custField(custIdMap.get("权重"),taskCustValueMap.get(custIdMap.get("新任务权重")).getJSONObject(0).getString("title"),""));
        array.add(custField(custIdMap.get("联席责任人"),taskCustValueMap.get(custIdMap.get("联席责任人"))));
        array.add(custField(custIdMap.get("行政上级领导"),taskCustValueMap.get(custIdMap.get("行政上级领导"))));
        array.add(custField(custIdMap.get("责任人"),taskCustValueMap.get(custIdMap.get("新任务责任人"))));
        array.add(custField(custIdMap.get("上级任务责任人"),taskCustValueMap.get(custIdMap.get("上级任务责任人"))));
        addTask.put("customfields",array);
        JSONObject result=tbApiService.addTask(addTask);
        log.info("新增完成 result：{}",result);
    }

    private JSONObject custField(String cfId,String title,String id){
        JSONObject result=new JSONObject();
        result.put("cfId",cfId);
        result.put("value",Arrays.asList(new JSONObject().fluentPut("title",title).fluentPut("id",id)));
        return result;
    }

    private JSONObject custField(String cfId,JSONArray value){
        JSONObject result=new JSONObject();
        result.put("cfId",cfId);
        result.put("value",value);
        return result;
    }

    public void update(String projectId,String taskId,String tfsName,String tfid){
        log.info("任务[{}]状态修改为[{}]",taskId,tfsName);
        JSONObject tfsnext=tbApiService.getProjectStateId(projectId,tfsName,tfid,"").getJSONObject(0);
        tbApiService.updateTaskStatus(taskId,tfsnext.getString("id"));
    }

    // 任务自定义字段变更
    public void taskCustChange(JSONObject task,JSONObject data){
        String projectId=task.getString("projectId");// 项目ID
        String customfieldId=data.getString("customfieldId");// 变更的自定义字段ID
        JSONArray array=data.getJSONArray("customfieldValue");// 负责人
        Map<String,String> projectCusts=new HashMap<>();
        List<JSONObject> list=tbApiService.getProjectCustomfield(projectId).toJavaList(JSONObject.class);
        list.forEach(item -> projectCusts.put(item.getString("name"),item.getString("id")));
        if(!customfieldId.equals(projectCusts.get("责任人"))){
            return;
        }
        String cid=projectCusts.get("上级任务责任人");
        JSONArray tasks=tbApiService.getTasksByParent(task.getString("id"));
        for (int i = 0; i < tasks.size(); i++) {
            JSONObject taskDetail=tasks.getJSONObject(i);
            String id=taskDetail.getString("id");
            tbApiService.updateTaskCustomfield(id,cid,array);
        }
    }

    // 变更任务创建
    public void creatChangeTask(JSONObject task,String name){
        String projectId=task.getString("projectId");// 项目ID
        // 查询项目自定义字段
        Map<String,String> projectCusts=new HashMap<>();
        List<JSONObject> list=tbApiService.getProjectCustomfield(projectId).toJavaList(JSONObject.class);
        list.forEach(item -> projectCusts.put(item.getString("name"),item.getString("id")));
        // 查询任务中的自定义字段值
        Map<String,JSONArray> taskCustValues=new HashMap<>();
        List<JSONObject> custs=task.getJSONArray("customfields").toJavaList(JSONObject.class);
        custs.forEach(item -> taskCustValues.put(item.getString("cfId"),item.getJSONArray("value")));
        String linkUrl=taskCustValues.get(projectCusts.get("原任务链接")).getJSONObject(0).getString("title");
        if(PublicUtil.isNull(linkUrl)){
           return;
        }
        String linkId=linkUrl.substring(linkUrl.indexOf("/task/")+6,linkUrl.indexOf("/task/")+30);
        JSONObject linkTask=tbApiService.getTasksInfo(linkId).getJSONObject(0);
        //添加关联任务
        tbApiService.addTaskLinks(task.getString("id"),linkTask.getString("content"),linkUrl,linkId);
        String custName;
        if(name.contains("二级变更流程")){
            custName="上级任务责任人";
        }else if(name.contains("三级变更流程")){
            linkTask=tbApiService.getTasksInfo(linkTask.getString("parentTaskId")).getJSONObject(0);
            custName="一级任务知晓人";
        }else if(name.contains("年度指标变更")){
            custName="二级任务责任人";
        }else{
            log.info("非期望任务类型 不处理");
            return;
        }
        List<JSONObject> linkCusts=linkTask.getJSONArray("customfields").toJavaList(JSONObject.class);
        String cfid=projectCusts.get("上级任务责任人");
        Optional optional = linkCusts.stream().filter(item -> cfid.equals(item.get("cfId"))).findAny();
        JSONArray value=((JSONObject) optional.get()).getJSONArray("value");
        tbApiService.updateTaskCustomfield(task.getString("id"),projectCusts.get(custName),value);
    }

    @Override
    public void syncState() {
        syncStateByTag(PROJECT_TAG_ID);
    }

    private void syncStateByTag(String tagId){
        JSONArray projectIds=tbApiService.getProjecTagUser(tagId);
        for (int j = 0; j < projectIds.size(); j++) {
            syncStateProject(projectIds.getString(j));
        }
        JSONArray array=tbApiService.getProjecGrouptTag(new JSONObject().fluentPut("parentProjectTagId",tagId));
        for (int i = 0; i < array.size(); i++) {
            syncStateByTag(array.getJSONObject(i).getString("id"));
        }
    }

    private String _matchFormUuid(String code) {
        Map<String, String> formUuid = UtilMap.empty();
        if (UtilEnv.getActiveProfile().equals(UtilEnv.ENV_PROD)) {
            formUuid.put("TASK", "FORM-B8C09F8900BD43EC8025068B3CB4C4F8SLAD");
            formUuid.put("CUSTFIELD", "FORM-031FC90C7D3C49A394600D4EE6C037915QSI");
        } else {
            formUuid.put("TASK", "FORM-59A439BCB8194F10B8C862C579F285F5KWG6");                // 文件夹/文件版本记录
            formUuid.put("CUSTFIELD", "FORM-C5B14D2EE4464AED835BA259388E829D9937");            // 文件夹权限
        }
        return formUuid.get(code);
    }

    private static String NULLDATA = "NULL";
    @Override
    public void syncStateProject(String projectId) {
        // 有责任人的任务编号
        List<Map> taskDatas = new ArrayList<>();
        // 同步项目下自定义字段
        Map<String,String> projectCusts=new HashMap<>();
        List<JSONObject> projectCoutList=tbApiService.getProjectCustomfield(projectId).toJavaList(JSONObject.class);
        projectCoutList.forEach(item -> {
            projectCusts.put(item.getString("name"),item.getString("id"));
        });
        // 同步项目下任务状态
        JSONObject project = tbApiService.getProjectInfo(projectId).getJSONObject(0);
        Map<String,String> map=getProjectStateVule(projectId);
        JSONArray array = getXiangmuRenwuLeixing(projectId);
        for (int j = 0; j < array.size(); j++) {
            String cTname=array.getJSONObject(j).getString("name");
            String cTid=array.getJSONObject(j).getString("id");
            if(cTname.contains("变更")){
                continue;
            }
            List<String> custNames;
            List<String> ydKyes;
            if(cTname.contains("一级")){
                custNames=Arrays.asList("责任人","联席责任人");
                ydKyes=Arrays.asList(NULLDATA,NULLDATA);
            }else if(cTname.contains("二级")){
                custNames=Arrays.asList("责任人","联席责任人","责任部门","上级任务责任人");
                ydKyes=Arrays.asList(NULLDATA,NULLDATA,"textField_m102rax6",NULLDATA);
            }else if(cTname.contains("三级")){
                custNames=Arrays.asList("责任人","联席责任人","责任部门","权重","上级任务责任人","问题与风险","解决方案思路");
                ydKyes=Arrays.asList(NULLDATA,NULLDATA,"textField_m102rax6","textField_m0xldyf7",NULLDATA,"textField_m21hlie0","textField_m21hlie1");
            }else if(cTname.contains("指标")){
                custNames=Arrays.asList("权重","统计口径","验收依据","Q1目标","Q1实际达成","Q1偏差原因说明","Q1改进举措","Q2目标","Q2实际达成","Q2偏差原因说明","Q2改进举措"
                        ,"Q3目标","Q3实际达成","Q3偏差原因说明","Q3改进举措","Q4目标","Q4实际达成","Q4偏差原因说明","Q4改进举措","年度指标完成情况","自评分","证明材料");
                ydKyes=Arrays.asList("textField_m0xldyf7","textField_m21hliea","textField_m102rax9",
                        "textField_m102raxm","textField_m21hlief","textField_m21hlieg","textField_m21hlieh", // q1
                        "textField_m10at4ej","textField_m21hliei","textField_m21hliel","textField_m21hlieo", // q2
                        "textField_m10at4ek","textField_m21hliej","textField_m21hliem","textField_m21hliep", // q3
                        "textField_m10at4el","textField_m21hliek","textField_m21hlien","textField_m21hlieq", // q4
                        "textField_m21hlieb","textField_m21hliec","textField_m21hlied");
            }else{
                custNames=new ArrayList<>();
                ydKyes=new ArrayList<>();
            }
            projectCoutList.forEach(item -> {
                if(custNames.contains(item.getString("name"))){
                    ydClient.operateData(YDParam.builder().formUuid(_matchFormUuid("CUSTFIELD"))
                            .formDataJson(JSONObject.toJSONString(UtilMap.map("textField_m0vvz8ni, textField_m0vvz8nj, textField_m0vvz8nk, textField_m0vvz8nl"
                                    ,projectId,cTid,item.getString("name"),item.getString("id"))))
                            .searchCondition(JSONObject.toJSONString(Arrays.asList(new YDSearch("textField_m0vvz8nl",item.getString("id"),"自定义字段编号", YDSearch.Type.TEXT_FIELD,YDSearch.Operator.EQ)
                                ,new YDSearch("textField_m0vvz8nj",cTid,"任务类型编号", YDSearch.Type.TEXT_FIELD,YDSearch.Operator.EQ))))
                            .build(), YDConf.FORM_OPERATION.upsert);
                }
            });

            JSONArray tasks=tbApiService.getTasksByType(projectId,cTid);
            for (int i = 0; i < tasks.size(); i++) {
                JSONObject task = tasks.getJSONObject(i);
                log.info("同步任务，任务编号：[{}]进度:[{}/{}]",task.getString("id"),i+1,tasks.size());
                Map saveMap=UtilMap.map("textField_m0rkuewb, textField_m0rkuewd, textField_m0rkuewc, textField_m0rkuewe, selectField_m0rkuewk, textField_m0vtq84z, textField_m0vtq850"
                        ,task.getString("id"),task.getString("content")
                        ,projectId,project.getString("name"),map.get(task.getString("tfsId"))
                        ,cTid,cTname);
                saveMap.put("textField_m0xldyf3",task.getString("parentTaskId"));
                JSONArray ancestorIds=task.getJSONArray("ancestorIds");
                int size=ancestorIds.size();
                if(size>0){
                    saveMap.put("textField_m0xldyf8",ancestorIds.getString(size-1));
                    if(size>1){
                        saveMap.put("textField_m0xldyf9",ancestorIds.getString(size-2));
                        if(size>2){
                            saveMap.put("textField_m0xldyfa",ancestorIds.getString(size-3));
                        }
                    }
                }
                // 查询任务中的自定义字段值
                Map<String,JSONArray> taskCustValues=new HashMap<>();
                List<JSONObject> custs=task.getJSONArray("customfields").toJavaList(JSONObject.class);
                custs.forEach(item -> taskCustValues.put(item.getString("cfId"),item.getJSONArray("value")));
                saveMap.put("textField_m21p4jl8",String.join(",",ancestorIds.toJavaList(String.class)));
                saveMap.put("employeeField_m0xldyf4",Arrays.asList(tbApiService.getUserId(task.getString("executorId"),true)));
                saveMap.put("employeeField_m0xldyf5",tbApiService.getUserIds(getCust(taskCustValues.get(projectCusts.get("联席责任人")),"id"),true));
                // 责任人
                List<String> zrrList=tbApiService.getUserIds(getCust(taskCustValues.get(projectCusts.get("责任人")),"id"),true);
                if(zrrList!=null&&zrrList.size()>0){
                    taskDatas.add(UtilMap.map("taskId, parentTaskId, zrr",task.getString("id"),task.getString("parentTaskId"),zrrList));
                }
                saveMap.put("employeeField_m201jmmi",zrrList);
                saveMap.put("employeeField_m21hlidz",tbApiService.getUserIds(getCust(taskCustValues.get(projectCusts.get("上级任务责任人")),"id"),true));
                for (int k = 0; k < ydKyes.size(); k++) {
                    if(!ydKyes.get(k).equals(NULLDATA)){
                        saveMap.put(ydKyes.get(k),getCustOne(taskCustValues.get(projectCusts.get(custNames.get(k))),"title"));
                    }
                }

                ydClient.operateData(YDParam.builder().formUuid(_matchFormUuid("TASK"))
                        .formDataJson(JSONObject.toJSONString(saveMap))
                        .searchCondition(JSONObject.toJSONString(Arrays.asList(new YDSearch("textField_m0rkuewb",task.getString("id"),"taskid", YDSearch.Type.TEXT_FIELD,YDSearch.Operator.EQ))))
                        .build(), YDConf.FORM_OPERATION.upsert);
            }
        }
        log.info("TB任务同步完成，开始处理权限成员");
        for (Map taskData:taskDatas){
            String taskId=UtilMap.getString(taskData,"taskId");
            String parentTaskId=UtilMap.getString(taskData,"parentTaskId");
            List<String> zrr=UtilMap.getList(taskData,"zrr");
            // 父项目添加当前项目负责人权限
            if(!PublicUtil.isNull(parentTaskId)){
                addTaskAuthUser("textField_m0rkuewb",parentTaskId,zrr); // 更新父级
            }
            // 查询父级任务并修改权限数据
            addTaskAuthUser("textField_m21p4jl8",taskId,zrr); // 更新子级
        }
    }

    private void addTaskAuthUser(String key,String taskId,List<String> users){
        List<Map> list = (List<Map>) ydClient.queryData(YDParam.builder().formUuid(_matchFormUuid("TASK")).searchCondition(
                JSONObject.toJSONString(Arrays.asList(new YDSearch(key,taskId,"taskid", YDSearch.Type.TEXT_FIELD,YDSearch.Operator.LIKE)))
        ).build(), YDConf.FORM_QUERY.retrieve_list).getData();
        if(list!=null&&list.size()>0){
            for (int i = 0; i < list.size(); i++) {
                Map formData=UtilMap.getMap(list.get(i),"formData");
                String formInstanceId=UtilMap.getString(list.get(i),"formInstanceId");
                List<String> newUsers = UtilMap.getList(formData,"employeeField_m217qmy5_id");
                newUsers.addAll(users);
                ydClient.operateData(YDParam.builder().formInstId(formInstanceId).updateFormDataJson(
                        JSONObject.toJSONString(UtilMap.map("employeeField_m217qmy5",newUsers))
                ).build(), YDConf.FORM_OPERATION.update);
            }
        }
    }

    private void taskStatusChange(JSONObject task){
        try {
            Map<String,String> map=getProjectStateVule(task.getString("projectId"));
            Map saveMap=UtilMap.map("selectField_m0rkuewk",map.get(task.getString("tfsId")));
            ydClient.operateData(YDParam.builder().formUuid(_matchFormUuid("TASK"))
                    .formDataJson(JSONObject.toJSONString(saveMap))
                    .searchCondition(JSONObject.toJSONString(Arrays.asList(new YDSearch("textField_m0rkuewb",task.getString("id"),"taskid", YDSearch.Type.TEXT_FIELD,YDSearch.Operator.EQ))))
                    .build(), YDConf.FORM_OPERATION.upsert);
        }catch (Exception e){
            log.error("同步到宜搭出错，不影响TB");
            e.printStackTrace();
        }
    }

    private String getCustOne(JSONArray array,String key){
        if(array==null||array.size()<1){
            return "";
        }else {
            return array.getJSONObject(0).getString(key);
        }
    }

    private String getCust(JSONArray array,String key){
        if(array==null||array.size()<1){
            return "";
        }else {
            return String.join(",",array.toJavaList(Map.class).stream().map(item->{
                return UtilMap.getString(item,key);
            }).collect(Collectors.toList()));
        }
    }

    @Override
    public void taskLock(String projectId, String taskType, List<String> keys, Boolean isAdd) {
        JSONArray tasks=tbApiService.getTasksByType(projectId,taskType);
        // 使用流来转换List<String>到List<Map<String, String>>
        List<Map<String, String>> mapList = keys.stream()
                .map(name -> Collections.singletonMap("action", name))
                .collect(Collectors.toList());

        for (int i = 0; i < tasks.size(); i++) {
            JSONObject task = tasks.getJSONObject(i);
            log.info(isAdd?"锁定":"解锁"+"任务字段，任务编号：[{}]进度:[{}/{}]",task.getString("id"),i+1,tasks.size());
            tbApiService.updateTaskLock(task.getString("id"),isAdd,mapList);
        }
    }

    @Override
    public McR taskUpdate(JSONObject param) {
        String type=param.getString("type");
        String taskId=param.getString("taskId");
        if(type.equals("content")){
            // 标题
            tbApiService.updateTaskLock(taskId,false,Arrays.asList(UtilMap.map("action","content$update")));
            tbApiService.updateTaskContent(taskId,param.getString("content"));
            tbApiService.updateTaskLock(taskId,true,Arrays.asList(UtilMap.map("action","content$update")));
        }else if(type.equals("executorId")){
            // 执行人
            tbApiService.updateTaskLock(taskId,false,Arrays.asList(UtilMap.map("action","executorId$update")));
            tbApiService.updateTaskExecutor(taskId,tbApiService.getUserId(param.getString("executorId"),false));
            tbApiService.updateTaskLock(taskId,true,Arrays.asList(UtilMap.map("action","executorId$update")));
        }else if(type.equals("qz")){
            // 权重
            String[] taskIds=taskId.split(",");
            String[] qzValues=param.getString("qzValues").split(",");
            if(taskIds.length!=qzValues.length){
                return McR.error("201","权重任务ID与数据长度不一致");
            }
            for (int i = 0; i < taskIds.length; i++) {
                tbApiService.updateTaskLock(taskIds[i],false,Arrays.asList(UtilMap.map("action","cf:"+param.getString("cId")+"$update")));
                tbApiService.updateTaskCustomfield(taskIds[i],param.getString("cId"),qzValues[i]);
                tbApiService.updateTaskLock(taskIds[i],false,Arrays.asList(UtilMap.map("action","cf:"+param.getString("cId")+"$update")));
            }
        }else if(type.equals("lxzrr")){
            // 联系责任人
            List<String> ids=param.getJSONArray("userIds").toJavaList(String.class);
            List<String> tbIds=tbApiService.getUserIds(String.join(",",ids),false);
            JSONArray array=new JSONArray();
            for (String id:tbIds){
                array.add(UtilMap.map("id",id));
            }
            tbApiService.updateTaskLock(taskId,false,Arrays.asList(UtilMap.map("action","cf:"+param.getString("cId")+"$update")));
            tbApiService.updateTaskCustomfield(taskId,param.getString("cId"),array);
            tbApiService.updateTaskLock(taskId,true,Arrays.asList(UtilMap.map("action","cf:"+param.getString("cId")+"$update")));
        }else if(type.equals("custValue")){
            // 自定义字段
            tbApiService.updateTaskLock(taskId,false,Arrays.asList(UtilMap.map("action","cf:"+param.getString("cId")+"$update")));
            tbApiService.updateTaskCustomfield(taskId,param.getString("cId"),param.getString("cValue"));
            tbApiService.updateTaskLock(taskId,true,Arrays.asList(UtilMap.map("action","cf:"+param.getString("cId")+"$update")));
        }else{
            return McR.error("201","type 错误");
        }
        return McR.success();
    }

    //获取项目任务类型
    public JSONArray getXiangmuRenwuLeixing(String xmid){
        JSONArray array=tbApiService.getTasksType(xmid);
        JSONArray result = new JSONArray();
        if (array != null){
            for (int i = 0; i < array.size(); i++) {
                Map map = new HashMap();
                String name = array.getJSONObject(i).getString("name");
                String id = array.getJSONObject(i).getString("id");
                map.put("name",name);
                map.put("id",id);
                result.add(map);
            }
        }
        return result;
    }

    private Map<String,String> getProjectStateVule(String projectId){
        Map<String,String> result=new HashMap<>();
        try {
            JSONArray array = tbApiService.getProjectStateId(projectId,"","","");
            List<JSONObject> list=array.toJavaList(JSONObject.class);
            list.forEach(item -> result.put(item.getString("id"),item.getString("name")));
            return result;
        }catch (Exception e){
            e.printStackTrace();
        }
        return result;
    }
}
